var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// index.ts
var esbuild_ifdef_next_exports = {};
__export(esbuild_ifdef_next_exports, {
  default: () => esbuild_ifdef_next_default
});
module.exports = __toCommonJS(esbuild_ifdef_next_exports);
var fs = __toESM(require("fs"));
var regExps = {
  double: /\/\/[\s]*#(?<token>.*?)(?:[\s]+(?<expression>.*?))?[\s]*$/,
  triple: /\/\/\/[\s]*#(?<token>.*?)(?:[\s]+(?<expression>.*?))?[\s]*$/
};
function ifdefPlugin(settings = {}) {
  const regExp = settings.regExp ?? (settings.requireTripleSlash !== false ? regExps.triple : regExps.double);
  const fileRegExp = settings.filePath ?? /\.[jt]sx?/;
  const variables = Object.freeze(settings.variables ?? { ...process.env });
  function getToken(line) {
    const match = line.match(regExp);
    if (match)
      return [
        match.groups.token,
        match.groups.expression ?? "",
        match.index,
        match[0].length
      ];
  }
  function evalExpression(expression, line, file) {
    const fn = new Function(
      ...Object.keys(variables),
      'return eval("' + expression + '")'
    );
    try {
      const res = !!fn(...Object.values(variables));
      if (settings.verbose)
        console.log(
          'Expression "' + expression + '" at ' + file + ":" + (line + 1) + " resulted with " + res
        );
      return res;
    } catch (e) {
      if (typeof e === "object")
        e.line = line;
      else {
        e = new Error("Error executing expression: " + e);
        e.line = line;
      }
      throw e;
    }
  }
  function parseIf(warn, file, lines, start = 0, ignore = false) {
    let remove = [];
    let prune = false;
    let done = false;
    let i;
    try {
      for (i = start; i < lines.length; i++) {
        const line = lines[i];
        const tokenData = getToken(line);
        if (prune || ignore)
          remove.push(i);
        if (!tokenData) {
          if (!prune && !ignore && settings.verbose)
            console.log("Including " + file + ":" + (i + 1));
          continue;
        }
        remove.push(i);
        const [token, expression, column, length] = tokenData;
        switch (token) {
          case "if": {
            if (i !== start) {
              const data = parseIf(warn, file, lines, i, prune || ignore);
              i = data.end;
              for (const n of data.remove)
                remove.push(n);
            } else {
              const exp = evalExpression(expression, i, file);
              done = exp;
              prune = !exp;
            }
            continue;
          }
          case "endif":
            return { end: i, remove };
        }
        if (ignore || prune && done)
          continue;
        switch (token) {
          case "else":
            prune = done;
            break;
          case "elseif":
          case "elif": {
            const exp = evalExpression(expression, i, file);
            prune = done || !exp;
            if (!done)
              done = exp;
          }
        }
        if (prune)
          continue;
        switch (token) {
          case "warning":
          case "warn":
            warn({
              text: expression,
              location: {
                line: i + 1,
                lineText: lines[i],
                column,
                length
              }
            });
            break;
          case "error":
          case "err":
            const err = new Error(expression);
            err.line = i;
            throw err;
        }
      }
      throw new Error("Unterminated #if found on line " + start);
    } catch (err) {
      const line = err.line ?? start;
      const tokenData = getToken(lines[line]);
      err.location = {
        line: line + 1,
        lineText: lines[line]
      };
      if (tokenData) {
        err.location.column = tokenData[2];
        err.location.length = tokenData[3];
      }
      throw err;
    }
  }
  function format(data, file, warn) {
    let i;
    let remove = [];
    const lines = data.split("\n");
    for (i = 0; i < lines.length; i++) {
      const line = lines[i];
      const tokenData = getToken(line);
      if (!tokenData || tokenData[0] !== "if")
        continue;
      const data2 = parseIf(warn, file, lines, i, false);
      i = data2.end;
      for (const n of data2.remove)
        remove.push(n);
    }
    let mapped;
    if (settings.fillWithSpaces)
      mapped = lines.map(
        (e, i2) => remove.includes(+i2) ? " ".repeat(e.length) : e
      );
    else
      mapped = lines.map((e, i2) => remove.includes(+i2) ? "//" + e : e);
    return mapped.join("\n");
  }
  const loaders = ["tsx", "jsx", "ts", "js"];
  return {
    name: "ifdef",
    setup(build) {
      build.onLoad({ filter: fileRegExp }, async (args) => {
        const warnings = [];
        try {
          const text = await fs.promises.readFile(args.path, "utf8");
          const formatted = format(
            text,
            args.path,
            (msg) => warnings.push({
              ...msg,
              location: { ...msg.location, file: args.path }
            })
          );
          const path = args.path.split(".");
          const ext = path[path.length - 1];
          return {
            contents: formatted,
            warnings,
            loader: loaders.includes(ext) ? ext : "js"
          };
        } catch (e) {
          if (!e.location)
            throw e;
          return {
            warnings,
            errors: [
              {
                text: e.message,
                detail: e,
                location: {
                  file: args.path,
                  ...e.location
                }
              }
            ]
          };
        }
      });
    }
  };
}
var esbuild_ifdef_next_default = ifdefPlugin;
